import subprocess
import time
from typing import Tuple
from voxl_utils_python.voxl_utils import is_voxl, _print

# Toggle voxl-px4 on or off
def toggle_voxl_px4(enabled:bool=None, running:bool=None) -> Tuple[bool, bool]:
	"""
	Toggle the state of voxl-px4. Needs to be off when interactive with the receiver.

	@running: Running state of voxl-px4
	@return: Returns the state of voxl-px4.
	"""
	
	start_voxl_px4_command = ['systemctl', 'start', 'voxl-px4']
	stop_voxl_px4_command = ['systemctl', 'stop', 'voxl-px4']
	
	# Case when we want to just stop voxl-px4
	if running == None:
		if service_running("voxl-px4"):
			running = True
			subprocess.run(stop_voxl_px4_command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
		else:
			running = False
	elif running:
		subprocess.run(start_voxl_px4_command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
	elif running == False:
		pass

	# Wait a bit so we don't get messages like "WARNING: Received 92 bytes of data from SLPI while SLPI uart not in use (topic aggregation)"
	time.sleep(1)
	return enabled, running

# Check if drone is armed
def drone_armed() -> int:
	"""
	Determine whether the drone is armed or not.

	@return: Returns whether the drone is armed (1) or not (0)
	"""
	if is_voxl():
		commander_command = "px4-commander status"
	else:
		commander_command = "adb shell px4-commander status"

	result = subprocess.run(commander_command.split(), stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
	arm_state = result.stdout.decode().split("\n")[0].split(":")[-1].strip()
	if "Standby" in arm_state or "not running" in arm_state:
		return 0
	elif "Armed" in arm_state:
		return 1
	else:
		_print("Unable to determine commander state of drone.")
		return -1

# Check if a service is running
def service_running(service_name: str) -> bool:
    """
    Checks if a systemd service is running or not

    @service_name: name of service to check
    @return: True/False depending on service state
    """
    cmd = ['systemctl', 'is-active', service_name]
    result = subprocess.run(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    if result.returncode == 0:
        return True
    else:
        return False
    
def px4_running() -> bool:
	service_running("voxl-px4")
